<?php

namespace Drupal\tidy\Plugin\Tamper;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;

/**
 * Plugin to tamper with website addresses to make them valid URLs.
 *
 * @Tamper(
 *   id = "tidy_url",
 *   label = @Translation("Tidy URL"),
 *   description = @Translation("Add 'http://' to domains and web paths that are missing them— make your forms and data input as forgiving as the address bar in browsers people are used to entering websites into."),
 *   category = "Text"
 * )
 */
class FixUrl extends TamperBase {

  const SETTING_HTTPS = 'https';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config[self::SETTING_HTTPS] = FALSE;
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form[self::SETTING_HTTPS] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use HTTPS'),
      '#default_value' => $this->getSetting(self::SETTING_HTTPS),
      '#description' => $this->t('If checked, "example.com" will be made "https://example.com" instead of "http://example.com".  This is more secure but more likely to cause a broken link for a website that could otherwise be reached.  If you think there should be an option to test the URL and figure this automatically, please file an issue.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->setConfiguration([
      self::SETTING_HTTPS => $form_state->getValue(self::SETTING_HTTPS),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL) {
    if (!is_string($data) && !is_numeric($data)) {
      throw new TamperException('Input should be a string or numeric.');
    }

    // Maybe this should be or already is done elsewhere in Tamper?  We want to
    // ensure it is always trimmed of whitespace and not empty before we start
    // our work though.
    $data = trim($data);
    if (!$data) {
      return $data;
    }

    if (strpos($data, 'https://') !== FALSE || strpos($data, 'http://')) {
      // They have a valid URL, or at least with anything we can help with.
      return $data;
    }

    if (parse_url($data, PHP_URL_SCHEME)) {
      // They have some sort of protocol!  Maybe it would probably be good to do
      // a soft warning here but that's our next module…
      return $data;
    }

    if (strpos($data, '.') === FALSE) {
      // If this is never going to be a vaild URL, this is not our problem— let
      // regular validation handle it.
      return $data;
    }
    if (strpos($data, '@') !== FALSE) {
      // If this person is putting @ symbols in their URLs and this is not meant
      // to be an e-mail address, they can be handling the scheme themselves.
      return $data;
    }
    // TODO re: all above look up the correct way to bail from a tamper plugin.

    $scheme_prefix = $this->getSetting(self::SETTING_HTTPS) ? 'https://' : 'http://';

    return $scheme_prefix . $data;
  }

}
