<?php

namespace Drupal\tidy\Plugin\Tamper;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;

/**
 * Plugin to tamper with text values to blank out self-professed 'not applicable' ones.
 *
 * @Tamper(
 *   id = "tidy_na_means_na",
 *   label = @Translation("Tidy Remove Not Applicable"),
 *   description = @Translation("Remove 'n/a' and similar from text fields."),
 *   category = "Text"
 * )
 */
class RemoveNotApplicable extends TamperBase {

  /**
   * If true, "NA" === "NA".
   */
  const CASE_SENSITIVE = 'case_sensitive';

  /**
   * If true, strip whitespace from all values, not only for the purpose of
   */
  const TRIM = 'trim';

  /**
   * A list of words/phrases to replace with blank entries.
   */
  const WORDS = 'words';

  /**
   * Index for the word list converted to an array.
   */
  const WORD_LIST = 'word_list';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config[self::WORDS] = 'N/A
Not Applicable
NA
N.A.';
    $config[self::CASE_SENSITIVE] = FALSE;
    $config[self::TRIM] = TRUE;
    $config[self::WORD_LIST] = [];

    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form[self::WORDS] = [
      '#type' => 'textarea',
      '#title' => $this->t('Values to skip'),
      '#default_value' => $this->getSetting(self::WORDS),
      '#description' => $this->t('A list of words/phrases that should be deleted rather than saved or imported.  Enter one value per line.'),
    ];

    $form[self::CASE_SENSITIVE] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Case sensitive'),
      '#default_value' => $this->getSetting(self::CASE_SENSITIVE),
      '#description' => $this->t('If checked, then "not applicable" will only match "not applicable" but not "Not Applicable" or "NOT APPLICABLE".'),
    ];

    $form[self::TRIM] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Trim all values'),
      '#default_value' => $this->getSetting(self::TRIM),
      '#description' => $this->t('If checked, remove preceeding and trailing blank spaces from values that are returned.  (Whitespace is always stripped from input before testing if it is a value to remove; this option is about stripping it from surviving input as well.)'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->setConfiguration([
      self::WORDS => $form_state->getValue(self::WORDS),
      self::CASE_SENSITIVE => $form_state->getValue(self::CASE_SENSITIVE),
      self::TRIM => $form_state->getValue(self::TRIM),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    $words = str_replace("\r", '', $form_state->getValue(self::WORDS));
    $word_list = explode("\n", $form_state->getValue(self::WORDS));
    $word_list = array_map('trim', $word_list);
    // Remove empty words from the list.
    $word_list = array_filter($word_list);

    $form_state->setValue(self::WORDS, $words);
    $form_state->setValue(self::WORD_LIST, $word_list);
  }

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL) {
    if (!is_string($data) && !is_numeric($data)) {
      throw new TamperException('Input should be a string or numeric.');
      // @TODO apparently tamper plugins should be able to handle arrays too?!
    }

    // Maybe this should be or already is done elsewhere in Tamper?  We want to
    // ensure it is always trimmed of whitespace and not empty before we start
    // our work though.
    $test = trim($data);
    if (!$test) {
      if ($this->getSetting(self::TRIM)) {
        return '';
      }
      else {
        return $data;
      }
    }

    if (!$this->getSetting(self::CASE_SENSITIVE)) {
      $test = strtolower($test);
    }

    $word_list = $this->getSetting(self::WORD_LIST);
    foreach ($word_list as $word) {
      if (!$this->getSetting(self::CASE_SENSITIVE)) {
        $word = strtolower($word);
      }
      if ($word === $test) {
        return '';
      }
    }
    if ($this->getSetting(self::TRIM)) {
      return $test;
    }
    else {
      return $data;
    }
  }

}
